#!/bin/sh

###########################################################
#
# @file: config/saga-install.sh(.in)
#
# @description:
#   used to install SAGA components into the target location.
#   The target location is either specified at configure time, 
#   via --prefix, or at runtime, via the SAGA_LOCATION 
#   environment variable, which overrides the configure prefix.
#
#   The script logs all activities to $SAGA_ROOT/install.log, 
#   which allows for later 'make uninstall'
#
# Copyright (c) 2009 Andre Merzky (andre@merzky.net)
# This file is part of the SAGA Engine.
#
# Contributed by Andre Merzky   <andre@merzky.net>
#
###########################################################

### LICENSE ###

# debug=yes

lock ()
{
  TEMPFILE="$1.$$"
  LOCKFILE="$1.lock"

  ( echo $$ > $TEMPFILE ) > /dev/null 2>&1 || {
    echo "You don't have permission to access `dirname $TEMPFILE`"
    return 1
  }

  ln $TEMPFILE $LOCKFILE  > /dev/null 2>&1 && {
    rm -f $TEMPFILE
    return 0
  }
  
  kill -0 `cat $LOCKFILE 2>> /dev/null` \
                          > /dev/null 2>&1 && {
    rm -f $TEMPFILE
    return 1
  }
  
  # echo "Removing stale lock file"
  rm -f $LOCKFILE         > /dev/null 2>&1
  ln $TEMPFILE $LOCKFILE  > /dev/null 2>&1 && {
    rm -f $TEMPFILE
    return 0
  }
  
  rm -f $TEMPFILE
  return 1
}


unlock ()
{
  LOCKFILE="$1.lock"

  rm -f $LOCKFILE
}


usage ()
{
  cat <<EOF

  Usage:    saga-install.sh    <component> [...] <path/to/dir/>
            saga-install.sh -f <component>       <path/to/file>
            saga-install.sh -d <component>       <path/to/dir>

  Examples: saga-install.sh    saga.hpp          include/saga/         
            saga-install.sh -f saga.hpp          include/saga/saga.hpp 
            saga-install.sh -d docs/             share/saga/docs/

  The target directory will _always_ be prefixed by 
  SAGA_LOCATION, i.e. by the installation prefix.
  <component> can be a file or directory.  If '-f'
  is specified, it MUST be a file.  if '-d' is 
  specified, it MUST be a directory.

EOF
  exit $1
}


# make sure ^C/^D does not leave a log behind
trap unlock INT
trap unlock KILL


# get defaults from configure
SAGA_ROOT="/home/condor/execute/dir_2628/userdir/SAGA.lsu-cpp.engine"

SAGA_LOC="/home/condor/execute/dir_2628/userdir/SAGA.lsu-cpp.engine/etics-tmp/usr"
if test "x$SAGA_LOCATION" != "x"; then
  SAGA_LOC=$SAGA_LOCATION
fi

# get rid of trailing slash
SAGA_LOC=`echo $SAGA_LOC | sed -e 's|/$||g'`


# we need arguments to work
if test $# -lt 2;
then
  usage 1 1>&2
fi

# check for mode flags
mode=default

case $1 in
  -f )
    mode=file
    shift
    ;;

  -d )
    mode=dir
    shift
    ;;

#  * )
#    break
#    ;;

esac

if test $mode != "default"; then
  # only _exactly_ two args allowed 
  if test $# -ne 2;
  then
    usage 1 1>&2
  fi
fi


# after flags, we still need two arguments, at least
if test $# -lt 2;
then
  usage 1 1>&2
fi

# all activities are logged, for a later 'make uninstall'
log=$SAGA_ROOT/install.log

# lock the logfile
until lock $log ; do
  # sleep 1
  # one sec sleep is too long, really.  Could not think of anything better than
  # the one below to obtain a somewhat shorter sleep, which does not keep the fs
  # busy
  env > /dev/null 2>&1
done


# get src files and target dir
src=$1
shift

while test $# -gt 1; do
  src="$src $1"
  shift
done

tgt=$SAGA_LOC/$1
shift

# tgtdir depends on the mode ( -f/ )
if test $mode = "file"; then
  tgtdir=`dirname $tgt`
else
  tgtdir=$tgt
fi

# get rid of potential double slashes, for better sorting
tgtdir=`echo $tgtdir | sed -e 's|//|/|g'`
tgt=`   echo $tgt    | sed -e 's|//|/|g'`


# tgt MUST be a directory, and if it does not 
# exist, we create it.
mkdir -p $tgtdir && ( echo "d $tgtdir" >> $log )

# beautify log file (this one gets rid of dirs)
cat $log  | sort -k 2 | uniq > $log.$$ 
mv  $log.$$ $log


# need to ensure thet tgt is an absolute path
tgtdir=`(cd $tgtdir && pwd)`

# install all sources into 
case $mode in
  file )
    if test "x$debug" = "xyes"; then
      printf "      installing   %-37s to $tgt\n" `basename $src`
    fi
    cp -f  $src $tgt && ( echo "f $tgt" >> $log )
    ;;

  dir )
    if test "x$debug" = "xyes"; then
      printf "      installing   %-37s to $tgt\n" `basename $src`
    fi
    cp -rf $src/* $tgtdir/ && ( echo "D $tgtdir" >> $log )
    ;;

  default )
    for s in $src; do
      base=`basename $s`
      if test "x$debug" = "xyes"; then
        printf "      installing   %-37s to $tgt/$base\n" $base
      fi
      cp -f  $s $tgtdir && ( echo "f $tgtdir/$base" >> $log )
    done
    ;;
esac


# beautify log file (this one gets rid of files, too)
cat $log  | sort -k 2 | uniq > $log.$$ 
mv  $log.$$ $log

# unlock logfile
unlock $log

